@@ -1,8 +1,8 @@
 """Various masking tools for VapourSynth"""
 
-__version__ = '0.9.2'
+__version__ = '1.3.2'
 
-__author_name__, __author_email__ = 'Irrational Encoding Wizardry', 'wizards@encode.moe'
+__author_name__, __author_email__ = 'Setsugen no ao', 'setsugen@setsugen.dev'
 __maintainer_name__, __maintainer_email__ = __author_name__, __author_email__
 
 __author__ = f'{__author_name__} <{__author_email__}>'
