@@ -6,8 +6,8 @@ from typing import Any, Sequence
 from vsexprtools import ExprOp, norm_expr
 from vsrgtools import box_blur
 from vstools import (
-    ColorRange, FrameRangeN, FrameRangesN, FramesLengthError, Position, Size, check_variable, depth, get_peak_value,
-    normalize_seq, replace_ranges, vs
+    ColorRange, FrameRangeN, FrameRangesN, FramesLengthError, Position, Size,
+    check_variable, depth, inject_self, normalize_seq, replace_ranges, vs
 )
 
 __all__ = [
@@ -22,6 +22,12 @@ class GeneralMask(ABC):
     def get_mask(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
         ...
 
+    @inject_self.init_kwargs.clean
+    def apply_mask(
+        self, _clipa: vs.VideoNode, _clipb: vs.VideoNode, _ref: vs.VideoNode | None = None, /, **kwargs: Any
+    ) -> vs.VideoNode:
+        return _clipa.std.MaskedMerge(_clipb, self.get_mask(_ref or _clipa, **kwargs))
+
 
 class BoundingBox(GeneralMask):
     pos: Position
@@ -33,7 +39,7 @@ class BoundingBox(GeneralMask):
 
     def get_mask(self, ref: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:  # type: ignore[override]
         from .utils import squaremask
-        return squaremask(ref, self.size.x, self.size.y, self.pos.x, self.pos.y, self.invert, self.get_mask)
+        return squaremask(ref, self.size.x, self.size.y, self.pos.x, self.pos.y, self.invert, False, self.get_mask)
 
 
 class DeferredMask(GeneralMask):
@@ -64,12 +70,6 @@ class DeferredMask(GeneralMask):
         assert check_variable(clip, self.get_mask)
         assert check_variable(ref, self.get_mask)
 
-        if self.bound:
-            bm = self.bound.get_mask(ref, **kwargs)
-
-            if self.blur:
-                bm = box_blur(bm, 5, 5)
-
         if self.refframes:
             hm = ref.std.BlankClip(
                 format=ref.format.replace(color_family=vs.GRAY, subsampling_h=0, subsampling_w=0).id, keep=True
@@ -94,7 +94,12 @@ class DeferredMask(GeneralMask):
             )
 
         if self.bound:
-            return norm_expr([hm, bm], f'y {ExprOp.clamp(0, get_peak_value(hm), "x")} 0 ?')
+            bm = self.bound.get_mask(hm, **kwargs)
+
+            if self.blur:
+                bm = box_blur(bm, 5, 5)
+
+            return norm_expr([hm, bm], f'y {ExprOp.clamp(c="x")} 0 ?')
 
         return hm.std.Limiter()
 
