@@ -7,7 +7,7 @@ from typing import Any, Sequence
 
 from vstools import ColorRange, depth, vs
 
-from ._abstract import EdgeDetect, EuclidianDistance
+from ._abstract import EdgeDetect, EuclideanDistance
 
 __all__ = [
     'Matrix1D',
@@ -123,7 +123,7 @@ class Matrix1D(EdgeDetect, ABC):
     ...
 
 
-class TEdge(EuclidianDistance, Matrix1D):
+class TEdge(EuclideanDistance, Matrix1D):
     """(TEdgeMasktype=2) Avisynth plugin."""
 
     matrices = [
@@ -141,7 +141,7 @@ class TEdgeTedgemask(Matrix1D, EdgeDetect):
         return clip.tedgemask.TEdgeMask(threshold=0, type=2)  # type: ignore
 
 
-class SavitzkyGolay(EuclidianDistance, Matrix1D):
+class SavitzkyGolay(EuclideanDistance, Matrix1D):
     mode_types = ['h', 'v']
 
 
@@ -149,8 +149,8 @@ class SavitzkyGolayNormalise(SavitzkyGolay):
     def _preprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
         return depth(clip, 32)
 
-    def _postprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
-        return depth(clip, self._bits, range_in=ColorRange.FULL, range_out=ColorRange.FULL)
+    def _postprocess(self, clip: vs.VideoNode, input_bits: int | None = None) -> vs.VideoNode:
+        return depth(clip, input_bits, range_in=ColorRange.FULL, range_out=ColorRange.FULL)
 
     def _get_matrices(self) -> Sequence[Sequence[float]]:
         assert self.divisors
