@@ -1,13 +1,15 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from enum import Enum, auto
+from enum import Enum, IntFlag, auto
 from typing import Any, ClassVar, NoReturn, Sequence, TypeAlias
 
+from stgpytools import inject_kwargs_params
 from vsexprtools import ExprOp, ExprToken, norm_expr
 from vstools import (
-    ColorRange, CustomRuntimeError, CustomValueError, FuncExceptT, PlanesT, T, check_variable, core, get_lowest_values,
-    get_peak_value, get_peak_values, get_subclasses, inject_self, join, normalize_planes, plane, vs
+    ColorRange, CustomRuntimeError, CustomValueError, DitherType, FuncExceptT, KwargsT, PlanesT, T,
+    check_variable, core, depth, get_lowest_values, get_peak_value, get_peak_values, get_subclasses,
+    inject_self, join, normalize_planes, plane, scale_mask, vs
 )
 
 from ..exceptions import UnknownEdgeDetectError, UnknownRidgeDetectError
@@ -16,10 +18,12 @@ __all__ = [
     'EdgeDetect', 'EdgeDetectT',
     'RidgeDetect', 'RidgeDetectT',
 
-    'MatrixEdgeDetect', 'SingleMatrix', 'EuclidianDistance',
+    'MatrixEdgeDetect', 'SingleMatrix', 'EuclideanDistance', 'MagnitudeMatrix',
 
     'Max',
 
+    'MagDirection',
+
     'get_all_edge_detects',
     'get_all_ridge_detect',
 ]
@@ -30,6 +34,41 @@ class _Feature(Enum):
     RIDGE = auto()
 
 
+class MagDirection(IntFlag):
+    """Direction of magnitude calculation."""
+
+    N = auto()
+    NW = auto()
+    W = auto()
+    SW = auto()
+    S = auto()
+    SE = auto()
+    E = auto()
+    NE = auto()
+
+    ALL = N | NW | W | SW | S | SE | E | NE
+
+    AXIS = N | W | S | E
+    CORNERS = NW | SW | SE | NE
+
+    NORTH = N | NW | NE
+    WEAST = W | NW | SW
+    EAST = E | NE | SE
+    SOUTH = S | SW | SE
+
+    def select_matrices(self, matrices: Sequence[T]) -> Sequence[T]:
+        # In Python <3.11, composite flags are included in MagDirection's
+        # collection and iteration interfaces.
+        primary_flags = [
+            flag for flag in MagDirection if flag != 0 and flag & (flag - 1) == 0
+        ]
+        assert len(matrices) == len(primary_flags) and self
+
+        return [
+            matrix for flag, matrix in zip(primary_flags, matrices) if self & flag
+        ]
+
+
 class BaseDetect:
     @staticmethod
     def from_param(
@@ -91,7 +130,12 @@ class BaseDetect:
 class EdgeDetect(ABC):
     """Abstract edge detection interface."""
 
-    _bits: int
+    kwargs: KwargsT | None = None
+
+    def __init__(self, **kwargs: Any) -> None:
+        super().__init__()
+
+        self.kwargs = kwargs
 
     @classmethod
     def from_param(
@@ -106,10 +150,11 @@ class EdgeDetect(ABC):
         return BaseDetect.ensure_obj(cls, edge_detect, UnknownEdgeDetectError, [], func_except)
 
     @inject_self
+    @inject_kwargs_params
     def edgemask(
         self, clip: vs.VideoNode, lthr: float = 0.0, hthr: float | None = None, multi: float = 1.0,
-        clamp: bool | tuple[float, float] | list[tuple[float, float]] = False, planes: PlanesT = None,
-        **kwargs: Any
+        clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
+        planes: PlanesT | tuple[PlanesT, bool] = None, **kwargs: Any
     ) -> vs.VideoNode:
         """
         Makes edge mask based on convolution kernel.
@@ -131,15 +176,21 @@ class EdgeDetect(ABC):
         lthr: float = 0.0, hthr: float | None = None,
         multi: float = 1.0,
         clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
-        feature: _Feature = _Feature.EDGE, planes: PlanesT = None, **kwargs: Any
+        feature: _Feature = _Feature.EDGE, planes: PlanesT | tuple[PlanesT, bool] = None, **kwargs: Any
     ) -> vs.VideoNode:
         assert check_variable(clip, self.__class__)
 
-        self._bits = clip.format.bits_per_sample
         peak = get_peak_value(clip)
-        hthr = peak if hthr is None else hthr
+        hthr = 1.0 if hthr is None else hthr
 
-        planes = normalize_planes(clip, planes)
+        lthr = scale_mask(lthr, 32, clip)
+        hthr = scale_mask(hthr, 32, clip)
+
+        discard_planes = False
+        if isinstance(planes, tuple):
+            planes, discard_planes = planes
+
+        planes = normalize_planes(clip, planes)  # type: ignore
 
         wclip = plane(clip, planes[0]) if len(planes) == 1 else clip
 
@@ -155,12 +206,14 @@ class EdgeDetect(ABC):
         except Exception as e:
             raise CustomRuntimeError('There was an error processing the mask! Are you using an abstract class?') from e
 
-        mask = self._postprocess(mask)
+        mask = self._postprocess(mask, clip.format.bits_per_sample)
 
         if multi != 1:
             mask = ExprOp.MUL(mask, suffix=str(multi), planes=planes)
 
-        if lthr > 0 and hthr < peak:
+        if lthr == hthr:
+            mask = norm_expr(mask, f'x {hthr} >= {ExprToken.RangeMax} 0 ?', planes)
+        elif lthr > 0 and hthr < peak:
             mask = norm_expr(mask, f'x {hthr} > {ExprToken.RangeMax} x {lthr} < 0 x ? ?', planes)
         elif lthr > 0:
             mask = norm_expr(mask, f'x {lthr} < 0 x ?', planes)
@@ -179,7 +232,7 @@ class EdgeDetect(ABC):
 
         assert mask.format
 
-        if mask.format.num_planes != clip.format.num_planes:
+        if mask.format.num_planes != clip.format.num_planes and not discard_planes:
             return join({None: clip.std.BlankClip(color=[0] * clip.format.num_planes, keep=True), planes[0]: mask})
 
         return mask
@@ -191,7 +244,7 @@ class EdgeDetect(ABC):
     def _preprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
         return clip
 
-    def _postprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
+    def _postprocess(self, clip: vs.VideoNode, input_bits: int | None = None) -> vs.VideoNode:
         return clip
 
 
@@ -217,7 +270,7 @@ class MatrixEdgeDetect(EdgeDetect):
         y = _y(clip)
         xx = _x(x)
         yy = _y(y)
-        xy = _x(x)
+        xy = _y(x)
         return self._merge_ridge([xx, yy, xy])
 
     @abstractmethod
@@ -237,7 +290,7 @@ class MatrixEdgeDetect(EdgeDetect):
     def _get_mode_types(self) -> Sequence[str]:
         return self.mode_types if self.mode_types else ['s'] * len(self._get_matrices())
 
-    def _postprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
+    def _postprocess(self, clip: vs.VideoNode, input_bits: int | None = None) -> vs.VideoNode:
         if len(self.matrices[0]) > 9 or (self.mode_types and self.mode_types[0] != 's'):
             clip = clip.std.Crop(
                 right=clip.format.subsampling_w * 2 if clip.format and clip.format.subsampling_w != 0 else 2
@@ -245,6 +298,16 @@ class MatrixEdgeDetect(EdgeDetect):
         return clip
 
 
+class MagnitudeMatrix(MatrixEdgeDetect):
+    def __init__(self, mag_directions: MagDirection = MagDirection.ALL, **kwargs: Any) -> None:
+        super().__init__(**kwargs)
+
+        self.mag_directions = mag_directions
+
+    def _get_matrices(self) -> Sequence[Sequence[float]]:
+        return self.mag_directions.select_matrices(self.matrices)
+
+
 class RidgeDetect(MatrixEdgeDetect):
     @classmethod
     def from_param(  # type: ignore
@@ -261,11 +324,13 @@ class RidgeDetect(MatrixEdgeDetect):
     @inject_self
     def ridgemask(
         self, clip: vs.VideoNode, lthr: float = 0.0, hthr: float | None = None, multi: float = 1.0,
-        clamp: bool | tuple[float, float] | list[tuple[float, float]] = False, **kwargs: Any
-    ) -> vs.VideoNode | NoReturn:
+        clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
+        planes: PlanesT | tuple[PlanesT, bool] = None, **kwargs: Any
+    ) -> vs.VideoNode:
         """
         Makes ridge mask based on convolution kernel.
         The resulting mask can be thresholded with lthr, hthr and multiplied with multi.
+        Using a 32-bit float clip is recommended.
 
         :param clip:            Source clip
         :param lthr:            Low threshold. Anything below lthr will be set to 0
@@ -275,10 +340,24 @@ class RidgeDetect(MatrixEdgeDetect):
 
         :return:                Mask clip
         """
-        return self._mask(clip, lthr, hthr, multi, clamp, _Feature.RIDGE, **kwargs)
+        return self._mask(clip, lthr, hthr, multi, clamp, _Feature.RIDGE, planes, **kwargs)
 
     def _merge_ridge(self, clips: Sequence[vs.VideoNode]) -> vs.VideoNode:
-        return core.std.Expr(clips, 'x y * 2 * -1 * x dup * z dup * 4 * + y dup * + + sqrt x y + +')
+        return core.std.Expr(clips, 'x 2 pow z 2 pow 4 * + x y * 2 * - y 2 pow + sqrt x y + + 0.5 *')
+
+    def _preprocess(self, clip: vs.VideoNode) -> vs.VideoNode:
+        if len(self.matrices[0]) > 9 or (self.mode_types and self.mode_types[0] != 's'):
+            clip = clip.resize.Point(clip.width + 4, src_width=clip.width + 4)
+
+        return super()._preprocess(depth(clip, 32))
+
+    def _postprocess(self, clip: vs.VideoNode, input_bits: int | None = None) -> vs.VideoNode:
+        clip = depth(clip, input_bits, dither_type=DitherType.NONE)
+
+        if len(self.matrices[0]) > 9 or (self.mode_types and self.mode_types[0] != 's'):
+            return clip.std.Crop(right=4)
+
+        return super()._postprocess(clip, input_bits)
 
 
 class SingleMatrix(MatrixEdgeDetect, ABC):
@@ -289,7 +368,7 @@ class SingleMatrix(MatrixEdgeDetect, ABC):
         raise NotImplementedError
 
 
-class EuclidianDistance(MatrixEdgeDetect, ABC):
+class EuclideanDistance(MatrixEdgeDetect, ABC):
     def _merge_edge(self, clips: Sequence[vs.VideoNode]) -> vs.VideoNode:
         return core.std.Expr(clips, 'x x * y y * + sqrt')
 
@@ -332,7 +411,7 @@ def get_all_edge_detects(
     all_subclasses = {
         s for s in _all_subclasses()
         if s.__name__ not in {
-            'MatrixEdgeDetect', 'RidgeDetect', 'SingleMatrix', 'EuclidianDistance', 'Max',
+            'MatrixEdgeDetect', 'RidgeDetect', 'SingleMatrix', 'EuclideanDistance', 'MagnitudeMatrix', 'Max',
             'Matrix1D', 'SavitzkyGolay', 'SavitzkyGolayNormalise',
             'Matrix2x2', 'Matrix3x3', 'Matrix5x5'
         }
@@ -364,7 +443,7 @@ def get_all_ridge_detect(
     all_subclasses = {
         s for s in _all_subclasses()
         if s.__name__ not in {
-            'MatrixEdgeDetect', 'RidgeDetect', 'SingleMatrix', 'EuclidianDistance', 'Max',
+            'MatrixEdgeDetect', 'RidgeDetect', 'SingleMatrix', 'EuclideanDistance', 'MagnitudeMatrix', 'Max',
             'Matrix1D', 'SavitzkyGolay', 'SavitzkyGolayNormalise',
             'Matrix2x2', 'Matrix3x3', 'Matrix5x5'
         }
