@@ -1,17 +1,20 @@
 from __future__ import annotations
 
 from functools import partial
-from typing import Any, Sequence
+from typing import Any, Literal, Sequence, cast, overload
 
 from vsexprtools import ExprOp, ExprToken, norm_expr
 from vsrgtools import gauss_blur
 from vsrgtools.util import wmean_matrix
-from vstools import check_variable, core, get_peak_value, get_y, iterate, plane, scale_value, vs
+from vstools import (
+    ColorRange, CustomEnum, VSFunctionAllArgs, check_variable, core, depth, get_peak_value, get_y, iterate, plane,
+    scale_value, scale_mask, vs
+)
 
 from .details import multi_detail_mask
-from .edge import FDoGTCanny, Kirsch, Prewitt
-from .funcs import retinex
+from .edge import FDoGTCanny, Kirsch, MagDirection, Prewitt, PrewittTCanny
 from .morpho import Morpho
+from .spat_funcs import retinex
 from .types import Coordinates, GenericMaskT
 from .utils import normalize_mask
 
@@ -22,7 +25,9 @@ __all__ = [
 
     'tcanny_retinex',
 
-    'limited_linemask'
+    'limited_linemask',
+
+    'dre_edgemask'
 ]
 
 
@@ -36,7 +41,7 @@ def ringing_mask(
     assert check_variable(clip, ringing_mask)
 
     thmi, thma, thlimi, thlima = (
-        scale_value(t, 32, clip) for t in [thmi, thma, thlimi, thlima]
+        scale_mask(t, 32, clip) for t in [thmi, thma, thlimi, thlima]
     )
 
     edgemask = normalize_mask(credit_mask, plane(clip, 0), **kwargs).std.Limiter()
@@ -62,7 +67,8 @@ def luma_mask(clip: vs.VideoNode, thr_lo: float, thr_hi: float, invert: bool = T
     lo, hi = (peak, 0) if invert else (0, peak)
     inv_pre, inv_post = (peak, '-') if invert else ('', '')
 
-    thr_lo, thr_hi = scale_value(thr_lo, 32, clip), scale_value(thr_hi, 32, clip)
+    thr_lo = scale_value(thr_lo, 32, clip)
+    thr_hi = scale_value(thr_hi, 32, clip)
 
     return norm_expr(
         get_y(clip),
@@ -77,7 +83,7 @@ def luma_credit_mask(
 
     edge_mask = normalize_mask(edgemask, y, **kwargs)
 
-    credit_mask = norm_expr([edge_mask, y], f'y {scale_value(thr, 32, y)} > y 0 ? x min')
+    credit_mask = norm_expr([edge_mask, y], f'y {scale_mask(thr, 32, y)} > y 0 ? x min')
 
     if not draft:
         credit_mask = Morpho.maximum(credit_mask, iterations=4)
@@ -112,3 +118,57 @@ def limited_linemask(
         (tcanny_retinex(clip_y, s) for s in sigmas),
         (multi_detail_mask(clip_y, s) for s in detail_sigmas)
     )
+
+
+class _dre_edgemask(CustomEnum):
+    """Edgemask with dynamic range enhancement prefiltering."""
+
+    RETINEX = cast(VSFunctionAllArgs, object())
+    CLAHE = cast(VSFunctionAllArgs, object())
+
+    def _prefilter(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
+        if self is self.RETINEX:  # type: ignore
+            sigmas = kwargs.get('sigmas', [50, 200, 350])
+
+            return retinex(clip, sigmas, 0.001, 0.005)
+
+        if self is self.CLAHE:  # type: ignore
+            limit, tile = kwargs.get('limit', 0.0305), kwargs.get('tile', 5)
+
+            return depth(depth(clip, 16).ehist.CLAHE(scale_value(limit, 32, 16), tile), clip)
+
+        return clip
+
+    @overload
+    def __call__(  # type: ignore
+        self: Literal[RETINEX], src: vs.VideoNode, tsigma: float = 1, brz: float = 0.122,
+        *, sigmas: Sequence[float] = [50, 200, 350]
+    ) -> vs.VideoNode:
+        ...
+
+    @overload
+    def __call__(  # type: ignore
+        self: Literal[CLAHE], src: vs.VideoNode, tsigma: float = 1, brz: float = 0.122,
+        *, limit: float = 0.0305, tile: int = 5
+    ) -> vs.VideoNode:
+        ...
+
+    def __call__(self, src: vs.VideoNode, tsigma: float = 1, brz: float = 0.122, **kwargs: Any) -> vs.VideoNode:
+        luma = get_y(src)
+
+        dreluma = self._prefilter(luma, **kwargs)
+
+        tcanny = PrewittTCanny.edgemask(dreluma, sigma=tsigma, scale=1)
+        tcanny = Morpho.minimum(tcanny, coords=Coordinates.CORNERS)
+
+        kirsch = Kirsch(MagDirection.N | MagDirection.EAST).edgemask(luma)
+
+        add_clip = ExprOp.ADD(tcanny, kirsch)
+
+        if brz > 0:
+            add_clip = Morpho.binarize(add_clip, brz)
+
+        return ColorRange.FULL.apply(add_clip)
+
+
+dre_edgemask = _dre_edgemask.RETINEX
