@@ -1,26 +1,30 @@
 from __future__ import annotations
 
-from dataclasses import dataclass
+from abc import abstractmethod
+from dataclasses import dataclass, field
 from pathlib import Path
-from typing import Any
+from typing import Any, Type
 
-from vsexprtools import ExprOp, ExprToken, norm_expr
-from vskernels import Catrom
+from vsexprtools import ExprOp, ExprToken, expr_func, norm_expr
+from vskernels import Bilinear, Catrom, Point
 from vsrgtools.util import mean_matrix
+from vssource import IMWRI, Indexer
 from vstools import (
-    CustomOverflowError, FileNotExistsError, VSFunction, check_variable, core, depth, fallback, get_neutral_value,
-    get_neutral_values, get_y, insert_clip, iterate, normalize_ranges, replace_ranges, scale_8bit, scale_value, vs,
-    vs_object
+    ColorRange, CustomOverflowError, FileNotExistsError, FilePathType, FrameRangeN, FrameRangesN,
+    Matrix, VSFunction, check_variable, core, depth, fallback, get_lowest_value, get_neutral_value,
+    get_neutral_values, get_peak_value, get_y, iterate, normalize_ranges, replace_ranges,
+    scale_value, scale_delta, vs, vs_object
 )
 
 from .abstract import DeferredMask, GeneralMask
-from .edge import Sobel
+from .edge import SobelStd
 from .morpho import Morpho
-from .types import GenericMaskT
-from .utils import normalize_mask
+from .types import GenericMaskT, XxpandMode
+from .utils import max_planes, normalize_mask
 
 __all__ = [
-    'HardsubManual',
+    'CustomMaskFromFolder',
+    'CustomMaskFromRanges',
 
     'HardsubMask',
     'HardsubSignFades',
@@ -36,27 +40,23 @@ __all__ = [
 ]
 
 
+class _base_cmaskcar(vs_object):
+    clips: list[vs.VideoNode]
+
+    @abstractmethod
+    def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]:
+        ...
+
+    def __vs_del__(self, core_id: int) -> None:
+        self.clips.clear()
+
+
 @dataclass
-class HardsubManual(GeneralMask, vs_object):
-    path: str | Path
-    processing: VSFunction = core.lazy.std.Binarize  # type: ignore
+class CustomMaskFromClipsAndRanges(GeneralMask, _base_cmaskcar):
+    processing: VSFunction = field(default=core.lazy.std.Binarize, kw_only=True)
+    idx: Indexer | Type[Indexer] = field(default=IMWRI, kw_only=True)
 
-    def __post_init__(self) -> None:
-        if not (path := Path(self.path)).is_dir():
-            raise FileNotExistsError('"path" must be an existing path directory!', self.get_mask)
-
-        files = [file.stem for file in path.glob('*')]
-
-        self.clips = [
-            core.imwri.Read(file) for file in files
-        ]
-
-        self.ranges = [
-            (other[-1] if other else end, end)
-            for (*other, end) in (map(int, name.split('_')) for name in files)
-        ]
-
-    def get_mask(self, clip: vs.VideoNode, *args: Any) -> vs.VideoNode:  # type: ignore[override]
+    def get_mask(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
         assert check_variable(clip, self.get_mask)
 
         mask = clip.std.BlankClip(
@@ -64,18 +64,48 @@ class HardsubManual(GeneralMask, vs_object):
             keep=True, color=0
         )
 
-        for maskclip, (start_frame, end_frame) in zip(self.clips, self.ranges):
-            maskclip = maskclip.std.AssumeFPS(clip).resize.Point(format=mask.format.id)  # type: ignore
-            maskclip = self.processing(maskclip).std.Loop(end_frame - start_frame + 1)
+        matrix = Matrix.from_video(clip)
 
-            mask = insert_clip(mask, maskclip, start_frame)
+        for maskclip, mask_ranges in zip(self.clips, self.frame_ranges(clip)):
+            maskclip = Point.resample(
+                maskclip.std.AssumeFPS(clip), mask.format, matrix,
+                range_in=ColorRange.FULL, range=ColorRange.FULL
+            )
+            maskclip = self.processing(maskclip).std.Loop(mask.num_frames)
+
+            mask = replace_ranges(mask, maskclip, mask_ranges, **kwargs)
 
         return mask
 
-    def __vs_del__(self, core_id: int) -> None:
-        super().__vs_del__(core_id)
 
-        self.clips.clear()
+@dataclass
+class CustomMaskFromFolder(CustomMaskFromClipsAndRanges):
+    folder_path: FilePathType
+
+    def __post_init__(self) -> None:
+        if not (folder_path := Path(str(self.folder_path))).is_dir():
+            raise FileNotExistsError('"folder_path" must be an existing path directory!', self.get_mask)
+
+        self.files = list(folder_path.glob('*'))
+
+        self.clips = [self.idx.source(file, bits=-1) for file in self.files]
+
+    def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]:
+        return [
+            [(other[-1] if other else end, end)]
+            for (*other, end) in (map(int, name.stem.split('_')) for name in self.files)
+        ]
+
+
+@dataclass
+class CustomMaskFromRanges(CustomMaskFromClipsAndRanges):
+    ranges: dict[FilePathType, FrameRangeN | FrameRangesN]
+
+    def __post_init__(self) -> None:
+        self.clips = [self.idx.source(str(file), bits=-1) for file in self.ranges.keys()]
+
+    def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]:
+        return [normalize_ranges(clip, ranges) for ranges in self.ranges.values()]
 
 
 class HardsubMask(DeferredMask):
@@ -108,7 +138,7 @@ class HardsubMask(DeferredMask):
                 ExprOp.SUB.combine(masks[-1], self.get_mask(p, ref))
             )
             dehardsub_masks.append(
-                iterate(core.akarin.Expr([masks[-1]], f"x {thr} < 0 x ?"), core.std.Maximum, 4).std.Inflate()
+                iterate(expr_func([masks[-1]], f"x {thr} < 0 x ?"), core.std.Maximum, 4).std.Inflate()
             )
             partials_dehardsubbed.append(
                 partials_dehardsubbed[-1].std.MaskedMerge(p, dehardsub_masks[-1])
@@ -134,13 +164,17 @@ class HardsubSignFades(HardsubMask):
     highpass: float
     expand: int
     edgemask: GenericMaskT
+    expand_mode: XxpandMode
 
     def __init__(
-        self, *args: Any, highpass: float = 0.0763, expand: int = 8, edgemask: GenericMaskT = Sobel, **kwargs: Any
+        self, *args: Any, highpass: float = 0.0763, expand: int = 8, edgemask: GenericMaskT = SobelStd,
+        expand_mode: XxpandMode = XxpandMode.RECTANGLE,
+        **kwargs: Any
     ) -> None:
         self.highpass = highpass
         self.expand = expand
         self.edgemask = edgemask
+        self.expand_mode = expand_mode
 
         super().__init__(*args, **kwargs)
 
@@ -150,47 +184,53 @@ class HardsubSignFades(HardsubMask):
             for x in (clip, ref)
         )
 
-        highpass = scale_value(self.highpass, 32, clip)
+        highpass = scale_delta(self.highpass, 32, clip)
 
         mask = norm_expr(
             [clipedge, refedge], f'x y - {highpass} < 0 {ExprToken.RangeMax} ?'
         ).std.Median()
 
-        return Morpho.inflate(Morpho.maximum(mask, iterations=self.expand), iterations=4)
+        return max_planes(Morpho.inflate(Morpho.expand(mask, self.expand, mode=self.expand_mode), iterations=4))
 
 
 class HardsubSign(HardsubMask):
     """
     Hardsub scenefiltering helper using `Zastin <https://github.com/kgrabs>`_'s hardsub mask.
 
-    :param thr:     Binarization threshold, [0, 1] (Default: 0.06).
-    :param expand:  std.Maximum iterations (Default: 8).
-    :param inflate: std.Inflate iterations (Default: 7).
+    :param thr:             Binarization threshold, [0, 1] (Default: 0.06).
+    :param minimum:         std.Minimum iterations (Default: 1).
+    :param expand:          std.Maximum iterations (Default: 8).
+    :param inflate:         std.Inflate iterations (Default: 7).
+    :param expand_mode:     Specifies the XxpandMode used for mask growth (Default: XxpandMode.RECTANGLE).
     """
 
     thr: float
     minimum: int
     expand: int
     inflate: int
+    expand_mode: XxpandMode
 
     def __init__(
-        self, *args: Any, thr: float = 0.06, minimum: int = 1, expand: int = 8, inflate: int = 7, **kwargs: Any
+        self, *args: Any, thr: float = 0.06, minimum: int = 1, expand: int = 8, inflate: int = 7,
+        expand_mode: XxpandMode = XxpandMode.RECTANGLE,
+        **kwargs: Any
     ) -> None:
         self.thr = thr
         self.minimum = minimum
         self.expand = expand
         self.inflate = inflate
+        self.expand_mode = expand_mode
         super().__init__(*args, **kwargs)
 
     def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         hsmf = norm_expr([clip, ref], 'x y - abs')
-        hsmf = hsmf.resize.Point(format=clip.format.replace(subsampling_w=0, subsampling_h=0).id)  # type: ignore
+        hsmf = Bilinear.resample(hsmf, clip.format.replace(subsampling_w=0, subsampling_h=0))  # type: ignore
 
         hsmf = ExprOp.MAX(hsmf, split_planes=True)
 
         hsmf = Morpho.binarize(hsmf, self.thr)
         hsmf = Morpho.minimum(hsmf, iterations=self.minimum)
-        hsmf = Morpho.maximum(hsmf, iterations=self.expand)
+        hsmf = Morpho.expand(hsmf, self.expand, mode=self.expand_mode)
         hsmf = Morpho.inflate(hsmf, iterations=self.inflate)
 
         return hsmf.std.Limiter()
@@ -209,17 +249,16 @@ class HardsubLine(HardsubMask):
 
         expand_n = fallback(self.expand, clip.width // 200)
 
-        y_range = scale_8bit(clip, 219) if clip.format.sample_type == vs.INTEGER else 1
-        uv_range = scale_8bit(clip, 224) if clip.format.sample_type == vs.INTEGER else 1
-        offset = scale_8bit(clip, 16) if clip.format.sample_type == vs.INTEGER else 0
+        y_range = get_peak_value(clip) - get_lowest_value(clip)
+        uv_range = get_peak_value(clip, chroma=True) - get_lowest_value(clip, chroma=True)
 
-        uv_abs = ' abs ' if clip.format.sample_type == vs.FLOAT else f' {get_neutral_value(clip)} - abs '
+        uv_abs = f' {get_neutral_value(clip)} - abs '
         yexpr = f'x y - abs {y_range * 0.7} > 255 0 ?'
         uv_thr = uv_range * 0.8
         uvexpr = f'x {uv_abs} {uv_thr} < y {uv_abs} {uv_thr} < and 255 0 ?'
 
-        upper = y_range * 0.8 + offset
-        lower = y_range * 0.2 + offset
+        upper = scale_value(0.8, 32, clip)
+        lower = scale_value(0.2, 32, clip)
         mindiff = y_range * 0.1
 
         difexpr = f'x {upper} > x {lower} < or x y - abs {mindiff} > and 255 0 ?'
@@ -241,7 +280,7 @@ class HardsubLine(HardsubMask):
         mask = iterate(mask, core.std.Maximum, expand_n)
         mask = mask.std.Inflate().std.Inflate().std.Convolution(mean_matrix)
 
-        return depth(mask, clip, range_in=1, range_out=1)
+        return depth(mask, clip, range_in=ColorRange.FULL, range_out=ColorRange.FULL)
 
 
 class HardsubLineFade(HardsubLine):
@@ -277,9 +316,7 @@ class HardsubASS(HardsubMask):
 
     def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         ref = ref[0] * self.shift + ref if self.shift else ref
-        mask: vs.VideoNode = ref.sub.TextFile(  # type: ignore[attr-defined]
-            self.filename, fontdir=self.fontdir, blend=False
-        )[1]
+        mask = ref.sub.TextFile(self.filename, fontdir=self.fontdir, blend=False).std.PropToClip('_Alpha')
         mask = mask[self.shift:] if self.shift else mask
         mask = mask.std.Binarize(1)
         mask = iterate(mask, core.std.Maximum, 3)
@@ -314,6 +351,6 @@ def get_all_sign_masks(hrdsb: vs.VideoNode, ref: vs.VideoNode, signs: list[Hards
     )
 
     for sign in signs:
-        mask = replace_ranges(mask, ExprOp.ADD.combine(mask, sign.get_mask(hrdsb, ref)), sign.ranges)
+        mask = replace_ranges(mask, ExprOp.ADD.combine(mask, max_planes(sign.get_mask(hrdsb, ref))), sign.ranges)
 
     return mask.std.Limiter()
