@@ -2,12 +2,13 @@ from __future__ import annotations
 
 from typing import Any, Callable, Concatenate, Iterable
 
-from vsexprtools import ExprOp, aka_expr_available, norm_expr
+from vsexprtools import ExprOp, complexpr_available, norm_expr
 from vskernels import Bilinear, Kernel, KernelT
 from vsrgtools import box_blur, gauss_blur
 from vstools import (
-    CustomValueError, FrameRangeN, FrameRangesN, FuncExceptT, P, check_ref_clip, check_variable, check_variable_format,
-    core, depth, flatten, get_peak_value, insert_clip, replace_ranges, split, vs
+    CustomValueError, ColorRange, FrameRangeN, FrameRangesN, FuncExceptT, P, check_ref_clip, check_variable,
+    check_variable_format, core, depth, flatten, get_lowest_values, get_peak_values, insert_clip,
+    normalize_ranges, plane, replace_ranges, split, vs
 )
 
 from .abstract import GeneralMask
@@ -48,7 +49,7 @@ def _get_region_expr(
     right, bottom = right + 1, bottom + 1
 
     if isinstance(replace, int):
-        replace = f'x {replace}'
+        replace = f'{replace} x'
 
     if rel:
         return f'X {left} < X {right} > or Y {top} < Y {bottom} > or or {replace} ?'
@@ -57,7 +58,7 @@ def _get_region_expr(
 
 
 def region_rel_mask(clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0) -> vs.VideoNode:
-    if aka_expr_available:
+    if complexpr_available:
         return norm_expr(clip, _get_region_expr(clip, left, right, top, bottom, 0), force_akarin=region_rel_mask)
 
     return clip.std.Crop(left, right, top, bottom).std.AddBorders(left, right, top, bottom)
@@ -70,7 +71,7 @@ def region_abs_mask(clip: vs.VideoNode, width: int, height: int, left: int = 0,
         )
 
     if 0 in {clip.width, clip.height}:
-        if aka_expr_available:
+        if complexpr_available:
             return norm_expr(
                 clip, _get_region_expr(clip, left, left + width, top, top + height, 0, True),
                 force_akarin=region_rel_mask
@@ -83,6 +84,7 @@ def region_abs_mask(clip: vs.VideoNode, width: int, height: int, left: int = 0,
 
 def squaremask(
     clip: vs.VideoNode, width: int, height: int, offset_x: int, offset_y: int, invert: bool = False,
+    force_gray: bool = True,
     func: FuncExceptT | None = None
 ) -> vs.VideoNode:
     """
@@ -97,6 +99,7 @@ def squaremask(
     :param offset_y:    The location of the square, offset from the top of the frame.
     :param invert:      Invert the mask. This means everything *but* the defined square will be masked.
                         Default: False.
+    :param force_gray:  Whether to force using GRAY format or clip format.
     :param func:        Function returned for custom error handling.
                         This should only be set by VS package developers.
                         Default: :py:func:`squaremask`.
@@ -107,25 +110,47 @@ def squaremask(
 
     assert check_variable(clip, func)
 
-    mask_format = clip.format.replace(color_family=vs.GRAY, subsampling_w=0, subsampling_h=0)
+    mask_format = clip.format.replace(
+        color_family=vs.GRAY, subsampling_w=0, subsampling_h=0
+    ) if force_gray else clip.format
 
     if offset_x + width > clip.width or offset_y + height > clip.height:
         raise CustomValueError('mask exceeds clip size!')
 
-    if aka_expr_available:
-        base_clip = clip.std.BlankClip(None, None, mask_format.id, 1, color=0, keep=True)
-
-        mask = norm_expr(
-            base_clip, _get_region_expr(
-                clip, offset_x, clip.width - width - offset_x, offset_y, clip.height - height - offset_y,
-                'range_max x' if invert else 'x range_max'
-            ), force_akarin=func
+    if complexpr_available:
+        base_clip = clip.std.BlankClip(
+            None, None, mask_format.id, 1,
+            color=get_lowest_values(mask_format),
+            keep=True
         )
+        exprs = [
+            _get_region_expr(
+                base_clip, offset_x, clip.width - width - offset_x, offset_y, clip.height - height - offset_y,
+                'range_max x' if invert else 'x range_max'
+            )
+        ]
+
+        if mask_format.num_planes > 1:
+            for i in range(1, mask_format.num_planes):
+                p = plane(base_clip, i)
+                ratio_x = p.width / base_clip.width
+                ratio_y = p.height / base_clip.height
+                exprs.append(
+                        _get_region_expr(
+                        p,
+                        int(offset_x * ratio_x), int((clip.width - width - offset_x) * ratio_x),
+                        int(offset_y * ratio_y), int((clip.height - height - offset_y) * ratio_y),
+                        'range_max x' if invert else 'x range_max'
+                    )
+                )
+
+        mask = norm_expr(base_clip, tuple(exprs), force_akarin=func)
     else:
-        base_clip = clip.std.BlankClip(width, height, mask_format.id, 1, color=get_peak_value(clip), keep=True)
+        base_clip = clip.std.BlankClip(width, height, mask_format.id, 1, color=get_peak_values(mask_format), keep=True)
 
         mask = base_clip.std.AddBorders(
-            offset_x, clip.width - width - offset_x, offset_y, clip.height - height - offset_y
+            offset_x, clip.width - width - offset_x, offset_y, clip.height - height - offset_y,
+            [0] * mask_format.num_planes
         )
         if invert:
             mask = mask.std.Invert()
@@ -169,10 +194,7 @@ def replace_squaremask(
 
     assert check_variable(clipa, func) and check_variable(clipb, func)
 
-    mask = squaremask(clipb[0], *mask_params, invert, func)
-
-    if ranges is None:
-        ranges = [(None, None)]
+    mask = squaremask(clipb[0], *mask_params, invert, func=func)
 
     if isinstance(blur_sigma, int):
         mask = box_blur(mask, blur_sigma)
@@ -186,6 +208,11 @@ def replace_squaremask(
 
     merge = clipa.std.MaskedMerge(clipb, mask)
 
+    ranges = normalize_ranges(clipa, ranges)
+
+    if len(ranges) == 1 and ranges[0] == (0, clipa.num_frames - 1):
+        return merge
+
     return replace_ranges(clipa, merge, ranges)
 
 
@@ -225,7 +252,7 @@ def normalize_mask(
 
         mask = mask(clip, ref)
 
-    return depth(mask, clip)
+    return depth(mask, clip, range_in=ColorRange.FULL, range_out=ColorRange.FULL)
 
 
 def rekt_partial(
@@ -244,15 +271,15 @@ def rekt_partial(
 
     check_ref_clip(cropped, filtered, rekt_partial)
 
-    if aka_expr_available:
+    if complexpr_available:
         filtered = filtered.std.AddBorders(left, right, top, bottom)
 
         ratio_w, ratio_h = 1 << clip.format.subsampling_w, 1 << clip.format.subsampling_h
 
         vals = list(filter(None, [
-            ('X {left} > ' if left else None),
+            ('X {left} >= ' if left else None),
             ('X {right} < ' if right else None),
-            ('Y {top} > ' if top else None),
+            ('Y {top} >= ' if top else None),
             ('Y {bottom} < ' if bottom else None)
         ]))
 
